apiVersion: v1
kind: Pod
spec:
  hostNetwork: True
  containers:
    - image: "{{ swift_rsyncd_image_full }}"
      name: swift-rsyncd
      volumeMounts:
        - mountPath: {{ container_config_directory }}/
          name: swift-rsyncd-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs
    - image: "{{ swift_object_image_full }}"
      name: swift-object-server
      volumeMounts:
        - mountPath: /var/lib/kolla/swift/
          name: swift-config
          readOnly: true
        - mountPath: {{ container_config_directory }}/
          name: swift-object-server-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs
    - image: "{{ swift_object_image_full }}"
      name: swift-object-auditor
      volumeMounts:
        - mountPath: /var/lib/kolla/swift/
          name: swift-config
          readOnly: true
        - mountPath: {{ container_config_directory }}/
          name: swift-object-auditor-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs
    - image: "{{ swift_object_image_full }}"
      name: swift-object-replicator
      volumeMounts:
        - mountPath: /var/lib/kolla/swift/
          name: swift-config
          readOnly: true
        - mountPath: {{ container_config_directory }}/
          name: swift-object-replicator-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs
    - image: "{{ swift_object_image_full }}"
      name: swift-object-updater
      volumeMounts:
        - mountPath: /var/lib/kolla/swift/
          name: swift-config
          readOnly: true
        - mountPath: {{ container_config_directory }}/
          name: swift-object-updater-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs
    - image: "{{ swift_account_image_full }}"
      name: swift-object-expirer
      volumeMounts:
        - mountPath: /var/lib/kolla/swift/
          name: swift-config
          readOnly: true
        - mountPath: {{ container_config_directory }}/
          name: swift-object-expirer-config
          readOnly: true
        - mountPath: {{ swift_devices_mount_point }}
          name: swift-devices
        - mountPath: /etc/localtime
          name: etc-localtime
          readOnly: true
        - mountPath: /var/log/kolla/
          name: kolla-logs

  volumes:
    - name: swift-rsyncd-config
      configMap:
        name: swift-rsyncd-configmap
    - name: swift-object-server-config
      configMap:
        name: swift-object-server-configmap
    - name: swift-object-auditor-config
      configMap:
        name: swift-object-auditor-configmap
    - name: swift-object-replicator-config
      configMap:
        name: swift-object-replicator-configmap
    - name: swift-object-updater-config
      configMap:
        name: swift-object-updater-configmap
    - name: swift-object-expirer-config
      configMap:
        name: swift-object-expirer-configmap
    - name: swift-config
      hostPath:
        path: {{ node_config_directory }}/swift/
    - name: swift-devices
      hostPath:
        path: {{ swift_devices_mount_point }}
    - name: etc-localtime
      hostPath:
        path: /etc/localtime
    - name: kolla-logs
      emptyDir: {}
metadata:
   name: swift-object