apiVersion: v1
kind: ReplicationController
spec:
  replicas: {{ neutron_control_replicas }}
  selector:
      service: neutron
      type: control
  template:
    metadata:
      labels:
        service: neutron
        type: control
    spec:
      containers:
        - image: "{{ neutron_server_image_full }}"
          name: neutron-server
          ports:
          - containerPort: {{ neutron_server_port }}
          volumeMounts:
            - mountPath: {{ container_config_directory }}
              name: neutron-server-config
              readOnly: true
            - mountPath: /etc/localtime
              name: host-etc-localtime
              readOnly: true
            - mountPath: /var/log/kolla/
              name: kolla-logs
          env:
            - name: KOLLA_CONFIG_STRATEGY
              value: {{ config_strategy }}
      volumes:
        - name: neutron-server-config
          configMap:
            name: neutron-server-configmap
            items:
            - key: neutron.conf
              path: neutron.conf
            - key: config.json
              path: config.json
            - key: ml2-conf.ini
              path: ml2_conf.ini
        - name: host-etc-localtime
          hostPath:
            path: /etc/localtime
        - name: kolla-logs
          emptyDir: {}
metadata:
   name: neutron-control