apiVersion: v1
kind: ReplicationController
spec:
  replicas: {{ memcached_replicas }}
  selector:
      service: memcached
  template:
    metadata:
      labels:
        service: memcached
    spec:
      containers:
        - image: "{{ memcached_image_full }}"
          name: memcached
          volumeMounts:
            - mountPath: {{ container_config_directory }}
              name: memcached-config
          env:
            - name: KOLLA_CONFIG_STRATEGY
              value: {{ config_strategy }}
          ports:
            - containerPort: {{ memcached_port }}
              name: memcached
      volumes:
        - name: memcached-config
          configMap:
            name: memcached-configmap
metadata:
   name: memcached