apiVersion: v1
kind: Service
spec:
  ports:
    - port: {{ neutron_server_port }}
      protocol: TCP
      targetPort: {{ neutron_server_port }}
  selector:
    service: neutron
    type: control
metadata:
  name: neutron-server