{%- set resourceName = kolla_kubernetes.cli.args.service_name %}

apiVersion: v1
kind: PersistentVolumeClaim
metadata:
  name: {{ resourceName }}
{%- if storage_provider in ["aws"] %}
  annotations:
    volume.alpha.kubernetes.io/storage-class: experimental_can_be_anything_in_kubernetes_1_2
{%- endif %}
spec:
  accessModes:
    - ReadWriteOnce
  resources:
    requests:
      storage: {{ kolla_kubernetes.template.vars.size_in_gb }}
